/*
Google Music Frame 

Copyright 2011 Jiří Janoušek <janousek.jiri@gmail.com>

This program is free software: you can redistribute it and/or modify it 
under the terms of the GNU General Public License version 3, as published 
by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranties of 
MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



using Fenryxo.Logger;
using GMF;

namespace GMF.Helpers{
	
	/**
	 * Playback  control actions 
	 */
	public enum Actions{
		PLAY=0,
		PAUSE,
		PREVIOUS_SONG,
		NEXT_SONG,
		THUMBS_UP,
		THUMBS_DOWN,
		QUIT,
		COUNT;
		
		public static Actions[] all(){
			return { PLAY, PAUSE, PREVIOUS_SONG, NEXT_SONG, THUMBS_UP, THUMBS_DOWN, QUIT };
		}
		
		public string get_label() {
	        switch(this){
			/// Hepler command
			case PLAY: return _("Play");
			/// Hepler command
			case PAUSE: return _("Pause");
			/// Hepler command
			case PREVIOUS_SONG: return _("Previous song");
			/// Hepler command
			case NEXT_SONG: return _("Next song");
			/// Hepler command
			case THUMBS_UP: return  _("Thumbs Up");
			/// Hepler command
			case THUMBS_DOWN: return _("Thumbs Down");
			/// Hepler command
			case QUIT: return _("Quit");
			case COUNT: return "";
            default:
                assert_not_reached();
	        }
	    }
	    
	    public string get_icon(){
			switch(this){
			case PLAY: return "media-playback-start";
			case PAUSE: return "media-playback-pause";
			case PREVIOUS_SONG: return "media-skip-backward";
			case NEXT_SONG: return "media-skip-forward";
			case THUMBS_UP: return "gtk-go-up";
			case THUMBS_DOWN: return "gtk-go-down";
			case QUIT: return "gtk-quit";
			case COUNT: return "";
            default:
                assert_not_reached();
	        }
		}
	}
	
	public abstract class BaseHelper: GLib.Object{
		protected GMF.Application app;
		protected GMF.Player player;
		
		public BaseHelper(GMF.Application app, GMF.Player player){
			this.app = app;
			this.player = player;
			this.player.playback_state_changed.connect(() => {this.update_menu();});
			this.app.terminated.connect(this.remove_menu);
		}
		
		public void trigger_action(Actions action){
			switch(action){
				case(Actions.PLAY):
				case(Actions.PAUSE):
					this.player.toggle_play();
					break;
				case(Actions.PREVIOUS_SONG):
					this.player.previous_song();
					break;
				case(Actions.NEXT_SONG):
					this.player.next_song();
					break;
				case(Actions.THUMBS_UP):
					this.player.thumbs_up();
					break;
				case(Actions.THUMBS_DOWN):
					this.player.thumbs_down();
					break;
				case(Actions.QUIT):
					try{
						this.app.quit();
					}
					catch(Fenryxo.AppError e){
						log_error("Unable to quit: %s", e.message);
					}
					break;
				default:
					log_warning("Unknown dock menu item: %d", action);
					break;
			}
		}
		
		protected abstract void create_menu();
		protected abstract void update_menu();
		protected abstract void remove_menu();
	}
}
